/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import java.util.ArrayList;
import java.util.Iterator;

import com.vaadin.ui.AbstractComponent;
import com.vaadin.ui.AbstractSingleComponentContainer;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Component;
import com.vaadin.ui.ComponentContainer;
import com.vaadin.ui.HasComponents;
import com.vaadin.ui.SingleComponentContainer;
import com.xmlnamespace.panel.vaadin.shared.ui.OverlayWrapperServerRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.OverlayWrapperState;

/**
 * Wrapper component that allows overlaying of one component on top of another.
 */
public class OverlayWrapper extends AbstractComponent implements
        HasComponents {
    
    private OverlayWrapperServerRpc rpc = new OverlayWrapperServerRpc() {
        
        @Override
        public void setOverlayVisible(boolean visible) {
            getState().overlayVisible = visible;
        }
    };
    
    /**
     * 
     */
    public OverlayWrapper() {
        registerRpc(rpc);
    }

    /**
     * Sets the content of the wrapper.
     * 
     * @param content
     *            the new content
     */
    public void setContent(Component content) {

        // Check if old content matches the new content
        Component oldContent = getContent();
        if (oldContent == content) {
            return;
        }

        // Remove the old content
        if (oldContent != null && oldContent.getParent() == this) {
            oldContent.setParent(null);
        }

        // Add new content
        if (content != null) {

            //If the content is attached to something else remove it
            if (content.getParent() != null) {
                
                if (content.getParent() instanceof ComponentContainer || content.getParent() instanceof SingleComponentContainer) {
                    AbstractSingleComponentContainer.removeFromParent(content);
                } else {
                    content.setParent(null);
                }
            }
            
            content.setParent(this);
        }

        // Set new content
        getState().content = content;
        
        markAsDirty();
    }

    /**
     * Gets the content of the wrapper.
     * 
     * @return the content
     */
    public Component getContent() {
        return (Component) getState().content;
    }

    /**
     * Sets the overlay component.
     * 
     * @param overlay
     *            the new overlay
     */
    public void setOverlay(Component overlay) {
        
        // Check if old overlay matches the new overlay
        Component oldOverlay = getOverlay();
        if (oldOverlay == overlay) {
            return;
        }
        
        // Remove the old overlay
        if (oldOverlay != null && oldOverlay.getParent() == this) {
            oldOverlay.setParent(null);
        }
        
        // Add new overlay
        if (overlay != null) {
            
            if (overlay.getParent() instanceof ComponentContainer || overlay.getParent() instanceof SingleComponentContainer) {
                AbstractSingleComponentContainer.removeFromParent(overlay);
            } else {
                overlay.setParent(null);
            }
            
            overlay.setParent(this);
        }
        
        // Set new overlay
        getState().overlay = overlay;

        markAsDirty();
    }

    /**
     * Gets the overlay component.
     * 
     * @return the overlay
     */
    public Component getOverlay() {
        return (Component) getState().overlay;
    }

    /**
     * Sets if the overlay is visible or not.
     * 
     * @param overlayVisible
     *            true, if overlay is visible
     */
    public void setOverlayVisible(boolean overlayVisible) {
        getState().overlayVisible = overlayVisible;
        markAsDirty();
    }

    /**
     * Checks if the overlay is visible or not.
     * 
     * @return true, if overlay is visible
     */
    public boolean isOverlayVisible() {
        return getState().overlayVisible;
    }

    /**
     * Sets the overlay reference alignment point on the wrapper.
     * 
     * @param align
     *            the new align
     */
    public void setAlign(Alignment align) {
        getState().align = align.getBitMask();
        markAsDirty();
    }

    /**
     * Gets the overlay reference alignment point of the wrapper.
     * 
     * @return the align
     */
    public Alignment getAlign() {
        return new Alignment(getState().align);
    }

    /**
     * Sets the overlay reference alignment point on the overlay component.
     * 
     * @param overlayAlign
     *            the new overlay align
     */
    public void setOverlayAlign(Alignment overlayAlign) {
        getState().overlayAlign = overlayAlign.getBitMask();
        markAsDirty();
    }

    /**
     * Gets the overlay reference alignment point of the overlay component.
     * 
     * @return the overlay align
     */
    public Alignment getOverlayAlign() {
        return new Alignment(getState().overlayAlign);
    }

    /**
     * Sets the overlay x offset.
     * 
     * @param x
     *            the new x
     */
    public void setX(int x) {
        getState().x = x;
        markAsDirty();
    }

    /**
     * Gets the overlay x offset.
     * 
     * @return the x
     */
    public int getX() {
        return getState().x;
    }

    /**
     * Sets the overlay y offset.
     * 
     * @param y
     *            the new y
     */
    public void setY(int y) {
        getState().y = y;
        markAsDirty();
    }

    /**
     * Gets the overlay y offset.
     * 
     * @return the y
     */
    public int getY() {
        return getState().y;
    }
    
    public boolean isHideOnClick() {
        return getState().hideOnClick;
    }
    
    public void setHideOnClick(boolean hideOnClick) {
        getState().hideOnClick = hideOnClick;
        markAsDirty();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractComponent#getState()
     */
    @Override
    public OverlayWrapperState getState() {
        return (OverlayWrapperState) super.getState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.HasComponents#iterator()
     */
    @Override
    public Iterator<Component> iterator() {
        ArrayList<Component> list = new ArrayList<Component>();

        if (getContent() != null) {
            list.add(getContent());
        }

        if (getOverlay() != null) {
            list.add(getOverlay());
        }

        return list.iterator();
    }
   
}
