/*
 * Copyright 2013 Vaadin Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.vaadin.addon.portallayout.portal;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.vaadin.addon.portallayout.gwt.shared.portal.PortalLayoutState;
import org.vaadin.addon.portallayout.portlet.Portlet;

import com.vaadin.event.LayoutEvents.LayoutClickEvent;
import com.vaadin.event.LayoutEvents.LayoutClickListener;
import com.vaadin.event.LayoutEvents.LayoutClickNotifier;
import com.vaadin.server.Extension;
import com.vaadin.shared.Connector;
import com.vaadin.shared.EventId;
import com.vaadin.shared.ui.MarginInfo;
import com.vaadin.ui.AbstractLayout;
import com.vaadin.ui.Component;
import com.vaadin.ui.ComponentContainer;
import com.vaadin.ui.HasComponents;
import com.vaadin.ui.Layout.MarginHandler;

/**
 * Base class for Portal Layouts.
 */
public abstract class PortalBase extends AbstractLayout implements
        MarginHandler, HasComponents, LayoutClickNotifier {

    /**
     * 
     */
    private static final long serialVersionUID = -5061118763242183659L;

    /**
     * Constructs a {@link StackPortalLayout}.
     */
    public PortalBase() {
        setStyleName("v-portal-layout");
    }

    /**
     * Wraps the provided component into a {@link Portlet} instance.
     * @param c Component to be wrapped into a {@link Portlet}.
     * @return created {@link Portlet}.
     */
    public Portlet portletFor(Component c) {
        Portlet result = getOrCreatePortletForComponent(c);
        return result;
    }
    
    /**
     * Finds the correspondent {@link Portlet} and removes it.
     * 
     * @param portletContent
     *            Content Component.
     */
    public void removePortlet(Component portletContent) {
        removePortlet((Portlet) getState().contentToPortlet.remove(portletContent));

    }

    /**
     * Removes a {@link Portlet} from current layout.
     * 
     * @param portletContent
     *            {@link Portlet} to be removed.
     */
    public void removePortlet(Portlet portlet) {
        Component portletContent = (Component) portlet.getParent();
        getState().portlets().remove(portlet);
        fireComponentDetachEvent(portletContent);
        if (portletContent.getParent() == this) {
            portletContent.setParent(null);
        }
    }

    @Override
    public void setMargin(boolean enabled) {
        setMargin(new MarginInfo(enabled));
    }

    @Override
    public MarginInfo getMargin() {
        return new MarginInfo(getState().marginsBitmask);
    }

    @Override
    public void setMargin(MarginInfo marginInfo) {
        getState().marginsBitmask = marginInfo.getBitMask();
    }
    
    @Override
    public Iterator<Component> iterator() {
        CombinedIterator<Component> cIt = new CombinedIterator<Component>();
        cIt.addIterator(portletContentIterator());
        cIt.addIterator(portletHeaderIterator());
        return cIt;
    }

    protected Portlet getPortlet(Component c) {
        return (Portlet) getState().contentToPortlet.get(c);
    }
    
    protected void onTheft(Component component) {}
    
    /**
     * Finds or creates a Portlet based on the Component. In case there is no
     * Portlet existing for the given Component - a new Portlet is created, if
     * there is a Portlet but it belongs to a different PortalLayout, current
     * Portal steals the Component from it.
     * 
     * @param c
     *            content Component.
     * @return either Portlet instance from own mapping, other portlet's mapping
     *         or a newly created instance.
     */
    protected Portlet getOrCreatePortletForComponent(Component c) {
        Portlet result = (Portlet) getState().contentToPortlet.get(c);
        if (result != null) {
            return result;
        } else {
            for (Extension extension : c.getExtensions()) {
                if (extension instanceof Portlet) {
                    addPortletMapping(c, (Portlet) extension);
                    return (Portlet) extension;
                }
            }
        }
        result = new Portlet(c);
        addPortletMapping(c, result);
        return result;
    }

    private void addPortletMapping(Component c, Portlet result) {
        getState().contentToPortlet.put(c, result);
        getState().portlets().add(result);

        if (c instanceof ComponentContainer) {
            for (Component parent = this; parent != null; parent = parent.getParent()) {
                if (parent == c) {
                    throw new IllegalArgumentException("Component cannot be added inside it's own content");
                }
            }
        }
        c.setParent(null);
        c.setParent(this);
        fireComponentAttachEvent(c);
    }

    @Override
    public void addComponentAttachListener(ComponentAttachListener listener) {
        addListener(ComponentAttachEvent.class, listener,
                ComponentAttachListener.attachMethod);
    }

    @Override
    public void removeComponentAttachListener(ComponentAttachListener listener) {
        removeListener(ComponentAttachEvent.class, listener,
                ComponentAttachListener.attachMethod);
    }

    @Override
    public void addComponentDetachListener(ComponentDetachListener listener) {
        addListener(ComponentDetachEvent.class, listener,
                ComponentDetachListener.detachMethod);
    }

    @Override
    public void removeComponentDetachListener(ComponentDetachListener listener) {
        removeListener(ComponentDetachEvent.class, listener,
                ComponentDetachListener.detachMethod);
    }

    /**
     * Fires the component attached event. This should be called by the
     * addComponent methods after the component have been added to this
     * container.
     * 
     * @param component
     *            the component that has been added to this container.
     */
    protected void fireComponentAttachEvent(Component component) {
        fireEvent(new ComponentAttachEvent(this, component));
    }

    /**
     * Fires the component detached event. This should be called by the
     * removeComponent methods after the component have been removed from this
     * container.
     * 
     * @param component
     *            the component that has been removed from this container.
     */
    protected void fireComponentDetachEvent(Component component) {
        fireEvent(new ComponentDetachEvent(this, component));
    }

    @Override
    protected PortalLayoutState getState() {
        return (PortalLayoutState) super.getState();
    }
    

    protected Iterator<Component> portletContentIterator() {
        return new PortletContentIterator();
    }

    protected Iterator<Component> portletHeaderIterator() {
        return new PortletHeaderIterator();
    }

    @Override
    public void addLayoutClickListener(LayoutClickListener listener) {
        addListener(EventId.LAYOUT_CLICK_EVENT_IDENTIFIER,
                LayoutClickEvent.class, listener,
                LayoutClickListener.clickMethod);
    }

    /**
     * @deprecated As of 7.0, replaced by
     *             {@link #addLayoutClickListener(LayoutClickListener)}
     **/
    @Override
    @Deprecated
    public void addListener(LayoutClickListener listener) {
        addLayoutClickListener(listener);
    }

    @Override
    public void removeLayoutClickListener(LayoutClickListener listener) {
        removeListener(EventId.LAYOUT_CLICK_EVENT_IDENTIFIER,
                LayoutClickEvent.class, listener);
    }

    /**
     * @deprecated As of 7.0, replaced by
     *             {@link #removeLayoutClickListener(LayoutClickListener)}
     **/
    @Override
    @Deprecated
    public void removeListener(LayoutClickListener listener) {
        removeLayoutClickListener(listener);
    }

    @Override
    public int getComponentCount() {
        return getState().portlets().size();
    }

    private static final class CombinedIterator<T> implements Iterator<T>, Serializable {

        private final Collection<Iterator<? extends T>> iterators = new ArrayList<Iterator<? extends T>>();
        
        public void addIterator(Iterator<? extends T> iterator) {
            iterators.add(iterator);
        }

        @Override
        public boolean hasNext() {
            for (Iterator<? extends T> i : iterators) {
                if (i.hasNext()) {
                    return true;
                }
            }
            return false;
        }

        @Override
        public T next() {
            for (Iterator<? extends T> i : iterators) {
                if (i.hasNext()) {
                    return i.next();
                }
            }
            throw new NoSuchElementException();
        }

        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    
    /**
     * IteratorImplementation.
     */
    private final class PortletContentIterator implements Iterator<Component> {
        private final Iterator<Connector> wrappedIt = getState().portlets().iterator();
        
        @Override
        public void remove() {
            wrappedIt.remove();
        }
        
        @Override
        public Component next() {
            return (Component) wrappedIt.next().getParent();
        }
        
        @Override
        public boolean hasNext() {
            return wrappedIt.hasNext();
        }
    }
    
    private final class PortletHeaderIterator implements Iterator<Component> {
        
        private final Iterator<Component> headersIt;
        
        public PortletHeaderIterator() {
            List<Component> headers = new ArrayList<Component>();
            for (Connector portletConnector : getState().portlets()) {
                Portlet p = (Portlet) portletConnector;
                if (p.getHeaderComponent() != null) {
                    headers.add(p.getHeaderComponent());
                }
            }
            this.headersIt = headers.iterator();
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public Component next() {
            return headersIt.next();
        }
        
        @Override
        public boolean hasNext() {
            return headersIt.hasNext();
        }
    }

}
